###############################################################################
#
# Xylem P_crit functions
#
###############################################################################
"""
    xylem_p_crit(vc::WeibullSingle{FT}, f_st::FT) where {FT<:AbstractFloat}
    xylem_p_crit(vc::WeibullDual{FT}, f_st::FT) where {FT<:AbstractFloat}

Returns the relative hydraulic conductance, given
- `vc` Xylem vulnerability curve
- `st` Relative surface tension. If missing, vis = 1.
"""
function xylem_p_crit(
    vc::WeibullSingle{FT},
    f_st::FT = FT(1)
) where {FT<:AbstractFloat}
    @unpack b,c = vc;

    return -b * log( FT(1000) ) ^ (1 / c) * f_st
end




function xylem_p_crit(
    vc::WeibullDual{FT},
    f_st::FT = FT(1)
) where {FT<:AbstractFloat}
    @unpack b1,c1,b2,c2 = vc;

    p1 = -b1 * log( FT(1000) ) ^ (1 / c1) * f_st
    p2 = -b2 * log( FT(1000) ) ^ (1 / c2) * f_st

    return min(p1, p2)
end
